@@ -1,11 +1,11 @@
 /* -*-pgsql-c-*- */
 /*
- * $Header: /cvsroot/pgpool/pgpool/child.c,v 1.15 2006/05/11 03:00:07 y-asaba Exp $
+ * $Header: /cvsroot/pgpool/pgpool/child.c,v 1.21 2007/02/03 04:56:18 t-ishii Exp $
  *
  * pgpool: a language independent connection pool server for PostgreSQL 
  * written by Tatsuo Ishii
  *
- * Copyright (c) 2003-2006	PgPool Global Development Group
+ * Copyright (c) 2003-2007	PgPool Global Development Group
  *
  * Permission to use, copy, modify, and distribute this software and
  * its documentation for any purpose and without fee is hereby
@@ -27,6 +27,8 @@
 #include <sys/socket.h>
 #include <netinet/in.h>
 #include <sys/un.h>
+#include <arpa/inet.h>
+#include <netdb.h>
 #ifdef HAVE_NETINET_TCP_H
 #include <netinet/tcp.h>
 #endif
@@ -45,6 +47,7 @@
 #endif
 
 #include "pool.h"
+#include "pool_ip.h"
 
 #ifdef NONE_BLOCK
 static void set_nonblock(int fd);
@@ -68,6 +71,11 @@ static int exit_request;
 
 static int idle;		/* non 0 means this child is in idle state */
 
+extern int myargc;
+extern char **myargv;
+
+char remote_ps_data[NI_MAXHOST];		/* used for set_ps_display */
+
 /*
 * child main loop
 */
@@ -83,9 +91,13 @@ void do_child(int unix_fd, int inet_fd)
 	int connections_count = 0;	/* used if child_max_connections > 0 */
 	int first_ready_for_query_received;		/* for master/slave mode */
 	int found;
+	char psbuf[NI_MAXHOST + 128];
 
 	pool_debug("I am %d", getpid());
 
+	/* Identify myself via ps */
+	init_ps_display("", "", "", "");
+
 	/* set up signal handlers */
 	signal(SIGALRM, SIG_DFL);
 	signal(SIGTERM, die);
@@ -211,6 +223,29 @@ void do_child(int unix_fd, int inet_fd)
 			goto retry_startup;
 		}
 
+		if (pool_config.enable_pool_hba)
+		{
+			/*
+			 * do client authentication.
+			 * Note that ClientAuthentication does not return if frontend
+			 * was rejected; it simply terminates this process.
+			 */
+			frontend->protoVersion = sp->major;
+			frontend->database = strdup(sp->database);
+			if (frontend->database == NULL)
+			{
+				pool_error("do_child: strdup failed: %s\n", strerror(errno));
+				exit(1);
+			}
+			frontend->username = strdup(sp->user);
+			if (frontend->username == NULL)
+			{
+				pool_error("do_child: strdup failed: %s\n", strerror(errno));
+				exit(1);
+			}
+			ClientAuthentication(frontend);
+		}
+
 		/*
 		 * Ok, negotiaton with frontend has been done. Let's go to the next step.
 		 */
@@ -327,6 +362,12 @@ void do_child(int unix_fd, int inet_fd)
 
 		connected = 1;
 
+		/* show ps status */
+		sp = MASTER_CONNECTION(backend)->sp;
+		snprintf(psbuf, sizeof(psbuf), "%s %s %s idle",
+				 sp->database, sp->user, remote_ps_data);
+		set_ps_display(psbuf, false);
+
 		/* query process loop */
 		for (;;)
 		{
@@ -384,9 +425,13 @@ void do_child(int unix_fd, int inet_fd)
 					exit(1);
 					break;
 
-				/* fatal error occured. just exit myself... */
+				/*
+				 * kind mismatch fatal error occured.
+				 * notice that we need to detach secondary server
+				 * and just exit myself...
+				 */
 				case POOL_FATAL:
-					notice_backend_error(1);
+					notice_backend_error(0);
 					exit(1);
 					break;
 
@@ -480,8 +525,7 @@ static POOL_CONNECTION *do_accept(int unix_fd, int inet_fd, struct timeval *time
     fd_set	readmask;
     int fds;
 
-	struct sockaddr addr;
-	socklen_t addrlen;
+	SockAddr saddr;
 	int fd = 0;
 	int afd;
 	int inet = 0;
@@ -494,6 +538,11 @@ static POOL_CONNECTION *do_accept(int unix_fd, int inet_fd, struct timeval *time
 	struct timeval *timeoutval;
 	struct timeval tv1, tv2, tmback;
 
+	char remote_host[NI_MAXHOST];
+	char remote_port[NI_MAXSERV];
+
+	set_ps_display("wait for connection request", false);
+
 	FD_ZERO(&readmask);
 	FD_SET(unix_fd, &readmask);
 	if (inet_fd)
@@ -578,12 +627,13 @@ static POOL_CONNECTION *do_accept(int unix_fd, int inet_fd, struct timeval *time
 	 * Note that some SysV systems do not work here. For those
 	 * systems, we need some locking mechanism for the fd.
 	 */
-	addrlen = sizeof(addr);
+	memset(&saddr, 0, sizeof(saddr));
+	saddr.salen = sizeof(saddr.addr);
 
 #ifdef ACCEPT_PERFORMANCE
 	gettimeofday(&now1,0);
 #endif
-	afd = accept(fd, &addr, &addrlen);
+	afd = accept(fd, (struct sockaddr *)&saddr.addr, &saddr.salen);
 	if (afd < 0)
 	{
 		/*
@@ -603,8 +653,23 @@ static POOL_CONNECTION *do_accept(int unix_fd, int inet_fd, struct timeval *time
 		pool_log("cnt: %d atime: %ld", cnt, atime);
 	}
 #endif
+
 	pool_debug("I am %d accept fd %d", getpid(), afd);
 
+	pool_getnameinfo_all(&saddr, remote_host, remote_port);
+	snprintf(remote_ps_data, sizeof(remote_ps_data),
+			 remote_port[0] == '\0' ? "%s" : "%s(%s)",
+			 remote_host, remote_port);
+
+	set_ps_display("accept connection", false);
+
+	/* log who is connecting */
+	if (pool_config.log_connections)
+	{
+		pool_log("connection received: host=%s%s%s",
+				 remote_host, remote_port[0] ? " port=" : "", remote_port);
+	}
+
 	/* set NODELAY and KEEPALIVE options if INET connection */
 	if (inet)
 	{
@@ -633,6 +698,12 @@ static POOL_CONNECTION *do_accept(int unix_fd, int inet_fd, struct timeval *time
 		close(afd);
 		return NULL;
 	}
+
+	/* save ip addres for hba */
+	memcpy(&cp->raddr, &saddr, sizeof(SockAddr));
+	if (cp->raddr.addr.ss_family == 0)
+		cp->raddr.addr.ss_family = AF_UNIX;
+	
 	return cp;
 }
 
@@ -963,11 +1034,23 @@ static RETSIGTYPE close_idle_connection(int sig)
 	}
 }
 
+/*
+ * send frontend exiting messages to all connections.
+ * this is called when child life time expires or child max connections expires.
+ */
 static void send_frontend_exits(void)
 {
 	int i;
 	POOL_CONNECTION_POOL *p = pool_connection_pool;
 
+#ifdef HAVE_SIGPROCMASK
+	sigset_t oldmask;
+#else
+	int	oldmask;
+#endif
+
+	POOL_SETMASK2(&BlockSig, &oldmask);
+
 	for (i=0;i<pool_config.max_pool;i++, p++)
 	{
 		if (!MASTER_CONNECTION(p))
@@ -976,6 +1059,8 @@ static void send_frontend_exits(void)
 			continue;
 		pool_send_frontend_exits(p);
 	}
+
+	POOL_SETMASK(&oldmask);
 }
 
 static int send_params(POOL_CONNECTION *frontend, POOL_CONNECTION_POOL *backend)
