@@ -1,11 +1,11 @@
 /* -*-pgsql-c-*- */
 /*
- * $Header: /cvsroot/pgpool/pgpool/pool_process_query.c,v 1.33 2006/12/01 06:50:26 y-asaba Exp $
+ * $Header: /cvsroot/pgpool/pgpool/pool_process_query.c,v 1.41 2007/02/03 04:22:21 t-ishii Exp $
  *
  * pgpool: a language independent connection pool server for PostgreSQL 
  * written by Tatsuo Ishii
  *
- * Copyright (c) 2003-2006	PgPool Global Development Group
+ * Copyright (c) 2003-2007	PgPool Global Development Group
  *
  * Permission to use, copy, modify, and distribute this software and
  * its documentation for any purpose and without fee is hereby
@@ -133,6 +133,7 @@ static void reset_prepared_list(PreparedStatementList *p);
 static PreparedStatement *lookup_prepared_statement_by_statement(PreparedStatementList *p, const char *name);
 static PreparedStatement *lookup_prepared_statement_by_portal(PreparedStatementList *p, const char *name);
 static int send_deallocate(POOL_CONNECTION_POOL *backend, PreparedStatementList *p, int n);
+static char *normalize_prepared_stmt_name(const char *name);
 
 static POOL_CONNECTION_POOL_SLOT *slots[MAX_CONNECTION_SLOTS];
 
@@ -149,6 +150,7 @@ static PreparedStatementList prepared_list; /* prepared statement name list */
 static PreparedStatement *unnamed_statement = NULL;
 static PreparedStatement *unnamed_portal = NULL;
 static int is_drop_database(char *query);		/* returns non 0 if this is a DROP DATABASE command */
+static void query_ps_status(char *query, POOL_CONNECTION_POOL *backend);		/* show ps status */
 
 POOL_STATUS pool_process_query(POOL_CONNECTION *frontend, 
 							   POOL_CONNECTION_POOL *backend,
@@ -584,6 +586,9 @@ static POOL_STATUS Query(POOL_CONNECTION *frontend,
 		string = query;
 	}
 
+	/* show ps status */
+	query_ps_status(string, backend);
+
 	/* log query to log file if neccessary */
 	if (pool_config.log_statement)
 	{
@@ -620,8 +625,17 @@ static POOL_STATUS Query(POOL_CONNECTION *frontend,
 	/* process status reporting? */
 	if (strncasecmp(sq, string, strlen(sq)) == 0)
 	{
+		StartupPacket *sp;
+		char psbuf[1024];
+
 		pool_debug("process reporting");
 		process_reporting(frontend, backend);
+
+		/* show ps status */
+		sp = MASTER_CONNECTION(backend)->sp;
+		snprintf(psbuf, sizeof(psbuf), "%s %s %s idle",
+				 sp->database, sp->user, remote_ps_data);
+		set_ps_display(psbuf, false);
 		return POOL_CONTINUE;
 	}
 
@@ -665,10 +679,9 @@ static POOL_STATUS Query(POOL_CONNECTION *frontend,
 			return POOL_END;
 		}
 
-		pending_prepared_stmt->statement_name = strdup(name);
+		pending_prepared_stmt->statement_name = normalize_prepared_stmt_name(name);
 		pending_prepared_stmt->portal_name = NULL;
-		if (pending_prepared_stmt->portal_name == NULL ||
-			pending_prepared_stmt->statement_name == NULL)
+		if (pending_prepared_stmt->statement_name == NULL)
 		{
 			pool_error("SimpleForwardToBackend: strdup failed: %s", strerror(errno));
 			return POOL_END;
@@ -955,6 +968,9 @@ static POOL_STATUS Sync(POOL_CONNECTION *frontend,
 static POOL_STATUS ReadyForQuery(POOL_CONNECTION *frontend, 
 								 POOL_CONNECTION_POOL *backend, int send_ready)
 {
+	StartupPacket *sp;
+	char psbuf[1024];
+
 	/* if a transaction is started for insert lock, we need to close it. */
 	if (internal_transaction_started)
 	{
@@ -1037,6 +1053,16 @@ static POOL_STATUS ReadyForQuery(POOL_CONNECTION *frontend,
 #ifdef NOT_USED
 	return ProcessFrontendResponse(frontend, backend);
 #endif
+
+	sp = MASTER_CONNECTION(backend)->sp;
+	if (MASTER(backend)->tstate == 'T')
+		snprintf(psbuf, sizeof(psbuf), "%s %s %s idle in transaction", 
+				 sp->database, sp->user, remote_ps_data);
+	else
+		snprintf(psbuf, sizeof(psbuf), "%s %s %s idle", 
+				 sp->database, sp->user, remote_ps_data);
+	set_ps_display(psbuf, false);
+
 	return POOL_CONTINUE;
 }
 
@@ -1860,7 +1886,7 @@ static POOL_STATUS ProcessFrontendResponse(POOL_CONNECTION *frontend,
 
 	if (pool_read(frontend, &fkind, 1) < 0)
 	{
-		pool_error("ProcessFrontendResponse: failed to read kind from frontend. fronend abnormally exited");
+		pool_error("ProcessFrontendResponse: failed to read kind from frontend. frontend abnormally exited");
 		return POOL_ERROR;
 	}
 
@@ -2226,6 +2252,26 @@ static void process_reporting(POOL_CONNECTION *frontend, POOL_CONNECTION_POOL *b
 	status[i].desc = "number of queries in reset_query_list";
 	i++;
 
+	status[i].name = "log_statement";
+	snprintf(status[i].value, MAXVALLEN, "%d", pool_config.log_statement);
+	status[i].desc = "if true, print all statements to the log";
+	i++;
+
+	status[i].name = "log_connections";
+	snprintf(status[i].value, MAXVALLEN, "%d", pool_config.log_connections);
+	status[i].desc = "if true, print incoming connections to the log";
+	i++;
+
+	status[i].name = "log_hostname";
+	snprintf(status[i].value, MAXVALLEN, "%d", pool_config.log_hostname);
+	status[i].desc = "if true, resolve hostname for ps and log print";
+	i++;
+
+	status[i].name = "enable_pool_hba";
+	snprintf(status[i].value, MAXVALLEN, "%d", pool_config.enable_pool_hba);
+	status[i].desc = "if true, use pool_hba.conf for client authentication";
+	i++;
+
 	status[i].name = "server_status";
 
 	if (pool_config.server_status[0] == 0)
@@ -2620,7 +2666,6 @@ POOL_STATUS SimpleForwardToBackend(char kind, POOL_CONNECTION *frontend, POOL_CO
 	int len;
 	int sendlen;
 	char *p;
-	int name_len;
 	char *name = NULL;
 
 	if (pool_write(MASTER(backend), &kind, 1))
@@ -2659,14 +2704,12 @@ POOL_STATUS SimpleForwardToBackend(char kind, POOL_CONNECTION *frontend, POOL_CO
 	{
 		char *stmt;
 		
-		name_len = strlen(p) + 3;
-		name = malloc(name_len);
+		name = strdup(p);
 		if (name == NULL)
 		{
 			pool_error("SimpleForwardToBackend: malloc failed: %s", strerror(errno));
 			return POOL_END;
 		}
-		sprintf(name, "\"%s\"", p);
 
 		pending_prepared_stmt = malloc(sizeof(PreparedStatement));
 		if (pending_prepared_stmt == NULL)
@@ -2712,21 +2755,18 @@ POOL_STATUS SimpleForwardToBackend(char kind, POOL_CONNECTION *frontend, POOL_CO
 			stmt = unnamed_statement;
 		else
 		{
-			name = malloc(strlen(stmt_name) + 3);
+			name = strdup(stmt_name);
 			if (name == NULL)
 			{
-				pool_error("SimpleForwardToBackend: malloc failed: %s", strerror(errno));
+				pool_error("SimpleForwardToBackend: strdup failed: %s", strerror(errno));
 				return POOL_END;
 			}
 
-			sprintf(name, "\"%s\"", stmt_name);
 			stmt = lookup_prepared_statement_by_statement(&prepared_list, name);
 			free(name);
 		}
 
-		if (stmt == NULL)
-			free(name);
-		else if (*portal_name == '\0')
+		if (*portal_name == '\0')
 			unnamed_portal = stmt;
 		else
 		{
@@ -2738,14 +2778,12 @@ POOL_STATUS SimpleForwardToBackend(char kind, POOL_CONNECTION *frontend, POOL_CO
 	}
 	else if (kind == 'C' && *p == 'S' && *(p + 1))
 	{
-		name_len = strlen(p + 1) + 3;
-		name = malloc(name_len);
+		name = strdup(p+1);
 		if (name == NULL)
 		{
-			pool_error("SimpleForwardToBackend: malloc failed: %s", strerror(errno));
+			pool_error("SimpleForwardToBackend: strdup failed: %s", strerror(errno));
 			return POOL_END;
 		}
-		sprintf(name, "\"%s\"", p + 1);
 		pending_function = del_prepared_list;
 		pending_prepared_stmt = malloc(sizeof(PreparedStatement));
 		if (pending_prepared_stmt == NULL)
@@ -2754,7 +2792,7 @@ POOL_STATUS SimpleForwardToBackend(char kind, POOL_CONNECTION *frontend, POOL_CO
 			return POOL_END;
 		}
 
-		pending_prepared_stmt->statement_name = strdup(name);
+		pending_prepared_stmt->statement_name = normalize_prepared_stmt_name(name);
 		if (pending_prepared_stmt->statement_name == NULL)
 		{
 			pool_error("SimpleForwardToBackend: malloc failed: %s", strerror(errno));
@@ -3431,12 +3469,10 @@ static PreparedStatement *get_prepared_command_portal_and_statement(char *query)
 	memcpy(string, query, len);
 
 	stmt = malloc(sizeof(PreparedStatement));
-	stmt->statement_name = strdup(portal);
+	stmt->statement_name = normalize_prepared_stmt_name(portal);
 	stmt->portal_name = NULL;
 	stmt->prepared_string = string;
 
-	pool_debug("get_prepared_command_portal_and_statement: extracted portal name: %s  portal statement: %s", stmt->portal_name, stmt->prepared_string);
-
 	return stmt;
 }
 
@@ -3618,14 +3654,14 @@ static int send_deallocate(POOL_CONNECTION_POOL *backend, PreparedStatementList
 	if (p->cnt <= n)
 		return 1;
 	
-	len = strlen(p->stmt_list[n]->statement_name) + 12; /* "DEALLOCATE " + '\0' */
+	len = strlen(p->stmt_list[n]->statement_name) + 14; /* "DEALLOCATE \"" + "\"" + '\0' */
 	query = malloc(len);
 	if (query == NULL)
 	{
 		pool_error("send_deallocate: malloc failed: %s", strerror(errno));
 		exit(1);
 	}
-	sprintf(query, "DEALLOCATE %s", p->stmt_list[n]->statement_name);
+	sprintf(query, "DEALLOCATE \"%s\"", p->stmt_list[n]->statement_name);
 	if (Query(NULL, backend, query) != POOL_CONTINUE)
 	{
 		free(query);
@@ -3635,3 +3671,67 @@ static int send_deallocate(POOL_CONNECTION_POOL *backend, PreparedStatementList
 
 	return 0;
 }
+
+static char *normalize_prepared_stmt_name(const char *name)
+{
+	char *result;
+	int i, len;
+
+	len = strlen(name);
+
+	if (name[0] != '"' && name[len-1] != '"')
+	{
+		result = strdup(name);
+		if (result == NULL)
+			return result;
+		for (i = 0; i < len; i++)
+		{
+			if (isupper(result[i]))
+			{
+				result[i] += 32; /* convert to lower */
+			}
+		}
+	}
+	else
+	{
+		result = malloc(len - 1);
+		if (result == NULL)
+			return result;
+
+		result = memcpy(result, name+1, len-2);
+		result[len-1] = '\0';
+	}
+
+	return result;
+}
+
+static void query_ps_status(char *query, POOL_CONNECTION_POOL *backend)
+{
+	StartupPacket *sp;
+	char psbuf[1024];
+	int i;
+
+	/* skip comment */
+    query = skip_comment(query);
+
+	if (*query == '\0')
+		return;
+
+	sp = MASTER_CONNECTION(backend)->sp;
+	i = snprintf(psbuf, sizeof(psbuf), "%s %s %s ", sp->database, sp->user, remote_ps_data);
+
+	/* skip spaces */
+	while (*query && isspace(*query))
+		query++;
+
+	for (; i< sizeof(psbuf); i++)
+	{
+		if (!*query || isspace(*query))
+			break;
+
+		psbuf[i] = toupper(*query++);
+	}
+	psbuf[i] = '\0';
+
+	set_ps_display(psbuf, false);
+}
