@@ -1,11 +1,11 @@
 /* -*-pgsql-c-*- */
 /*
-* $Header: /cvsroot/pgpool/pgpool/pool_stream.c,v 1.7 2006/08/02 08:00:56 t-ishii Exp $
+* $Header: /cvsroot/pgpool/pgpool/pool_stream.c,v 1.9 2007/02/01 16:40:35 yamaguti Exp $
 *
 * pgpool: a language independent connection pool server for PostgreSQL 
 * written by Tatsuo Ishii
 *
-* Copyright (c) 2003-2006	PgPool Global Development Group
+* Copyright (c) 2003-2007	PgPool Global Development Group
 *
 * Permission to use, copy, modify, and distribute this software and
 * its documentation for any purpose and without fee is hereby
@@ -83,6 +83,12 @@ POOL_CONNECTION *pool_open(int fd)
 	cp->sbufsz = 0;
 
 	cp->fd = fd;
+
+	memset((void *) &cp->raddr, 0, sizeof(cp->raddr));
+	cp->auth_arg = NULL;
+	cp->database = NULL;
+	cp->username = NULL;
+
 	return cp;
 }
 
@@ -100,6 +106,12 @@ void pool_close(POOL_CONNECTION *cp)
 		free(cp->buf2);
 	if (cp->isbackend)
 		pool_discard_params(&cp->params);
+	if (cp->auth_arg)
+		free(cp->auth_arg);
+	if (cp->database)
+		free(cp->database);
+	if (cp->username)
+		free(cp->username);
 	free(cp);
 }
 
